<?php
/**
 * Updater
 *
 *  @package Live Composer - WooCommerce integration
 */

// Prevent direct access to the file.
if ( ! defined( 'ABSPATH' ) ) {
	header( 'HTTP/1.0 403 Forbidden' );
	exit;
}

/**
 * Creates settings in the options table.
 */
function lcwoo_edd_register_option() {
	register_setting( 'lcwoo_edd_license', 'lcwoo_edd_license_key', 'lcwoo_edd_sanitize_license' );
}
add_action( 'admin_init', 'lcwoo_edd_register_option' );

/**
 * A callback function that sanitizes the option's value
 *
 * @param string $new option value.
 */
function lcwoo_edd_sanitize_license( $new ) {
	$old = get_option( 'lcwoo_edd_license_key' );
	if ( $old && $old != $new ) {
		delete_option( 'lcwoo_edd_license_status' ); // New license has been entered, so must reactivate.
	}
	return $new;
}

/**
 * Activatea license key.
 */
function lcwoo_edd_activate_license() {

	// Listen for our activate button to be clicked.
	if ( isset( $_POST['lcwoo_edd_license_activate'] ) ) {

		// Run a quick security check.
	 	if ( ! check_admin_referer( 'lcwoo_edd_nonce', 'lcwoo_edd_nonce' ) ) {
			return; // Get out if we didn't click the Activate button.
	 	}

		// Retrieve the license from the database.
		$license = trim( get_option( 'lcwoo_edd_license_key' ) );

		// Data to send in our API request.
		$api_params = array(
			'edd_action' => 'activate_license',
			'license'    => $license,
			'item_name'  => urlencode( EDD_LCWOO_ITEM_NAME ), // The name of our product in EDD.
			'url'        => home_url(),
		);

		// Call the custom API.
		$response = wp_remote_post( EDD_LCWOO_STORE_URL, array( 'timeout' => 15, 'sslverify' => false, 'body' => $api_params ) );

		// Make sure the response came back okay.
		if ( is_wp_error( $response ) || 200 !== wp_remote_retrieve_response_code( $response ) ) {

			if ( is_wp_error( $response ) ) {
				$message = $response->get_error_message();
			} else {
				$message = __( 'An error occurred, please try again.' );
			}
		} else {

			$license_data = json_decode( wp_remote_retrieve_body( $response ) );

			if ( false === $license_data->success ) {

				switch ( $license_data->error ) {

					case 'expired' :

						$message = sprintf(
							__( 'Your license key expired on %s.' ),
							date_i18n( get_option( 'date_format' ), strtotime( $license_data->expires, current_time( 'timestamp' ) ) )
						);
						break;

					case 'revoked' :

						$message = __( 'Your license key has been disabled.' );
						break;

					case 'missing' :

						$message = __( 'Invalid license.' );
						break;

					case 'invalid' :
					case 'site_inactive' :

						$message = __( 'Your license is not active for this URL.' );
						break;

					case 'item_name_mismatch' :

						$message = sprintf( __( 'This appears to be an invalid license key for %s.' ), EDD_LCWOO_ITEM_NAME );
						break;

					case 'no_activations_left':

						$message = __( 'Your license key has reached its activation limit.' );
						break;

					default :

						$message = __( 'An error occurred, please try again.' );
						break;
				}
			}
		}

		// Check if anything passed on a message constituting a failure.
		if ( ! empty( $message ) ) {
			$base_url = admin_url( 'admin.php?page=' . EDD_LCWOO_LICENSE_PAGE );
			$redirect = add_query_arg( array( 'sl_activation' => 'false', 'message' => urlencode( $message ) ), $base_url );

			wp_redirect( $redirect );
			exit();
		}

		update_option( 'lcwoo_edd_license_status', $license_data->license );
		wp_redirect( admin_url( 'admin.php?page=' . EDD_LCWOO_LICENSE_PAGE ) );
		exit();
	}
}
add_action( 'admin_init', 'lcwoo_edd_activate_license' );

/**
 * Deactivate license key.
 */
function lcwoo_edd_deactivate_license() {

	// Listen for our activate button to be clicked.
	if ( isset( $_POST['lcwoo_edd_license_deactivate'] ) ) {

		// Run a quick security check.
	 	if ( ! check_admin_referer( 'lcwoo_edd_nonce', 'lcwoo_edd_nonce' ) ) {
			return; // Get out if we didn't click the Activate button.
	 	}

		// Retrieve the license from the database.
		$license = trim( get_option( 'lcwoo_edd_license_key' ) );

		// Data to send in our API request.
		$api_params = array(
			'edd_action' => 'deactivate_license',
			'license'    => $license,
			'item_name'  => urlencode( EDD_LCWOO_ITEM_NAME ), // The name of our product in EDD.
			'url'        => home_url(),
		);

		// Call the custom API.
		$response = wp_remote_post( EDD_LCWOO_STORE_URL, array( 'timeout' => 15, 'sslverify' => false, 'body' => $api_params ) );

		// Make sure the response came back okay.
		if ( is_wp_error( $response ) || 200 !== wp_remote_retrieve_response_code( $response ) ) {

			if ( is_wp_error( $response ) ) {
				$message = $response->get_error_message();
			} else {
				$message = __( 'An error occurred, please try again.' );
			}

			$base_url = admin_url( 'admin.php?page=' . EDD_LCWOO_LICENSE_PAGE );
			$redirect = add_query_arg( array( 'sl_activation' => 'false', 'message' => urlencode( $message ) ), $base_url );

			wp_redirect( $redirect );
			exit();
		}

		// Decode the license data.
		$license_data = json_decode( wp_remote_retrieve_body( $response ) );

		// $license_data->license will be either "deactivated" or "failed"
		if ( $license_data->license == 'deactivated' ) {
			delete_option( 'lcwoo_edd_license_status' );
		}

		wp_redirect( admin_url( 'admin.php?page=' . EDD_LCWOO_LICENSE_PAGE ) );
		exit();

	}
}
add_action( 'admin_init', 'lcwoo_edd_deactivate_license' );

/**
 * This is a means of catching errors from the activation method above and displaying it to the customer
 */
function lcwoo_edd_admin_notices() {
	if ( isset( $_GET['sl_activation'] ) && ! empty( $_GET['message'] ) ) {
		switch ( $_GET['sl_activation'] ) {
			case 'false':
				$message = urldecode( $_GET['message'] );
				?>
				<div class="error">
					<p><?php echo $message; ?></p>
				</div>
				<?php
				break;
			case 'true':
			default:
				// Developers can put a custom success message here for when activation is successful if they way.
				break;
		}
	}
}
add_action( 'admin_notices', 'lcwoo_edd_admin_notices' );

/**
 * Register Admin Settings
 */
function lcwoo_register_admin_settings() {

	global $dslc_options_extender;

	$array = [
		'title' => __( 'Woo - License', 'lc-woo-integration' ),
		'extension_id' => 'woo_license',
		'sections' => [
			[
				'id' => 'main',
				'title' => __( 'Woo - License', 'lc-woo-integration' ),
				'options' => '',
			],
		],
	];

	$dslc_options_extender->add_settings_panel( $array );
}
add_action( 'dslc_extend_admin_panel_options', 'lcwoo_register_admin_settings' );

/**
 * Add Settings Description
 *
 * @param HTML   $description output HTML.
 * @param string $pannel_id panel id.
 */
function lcwoo_add_settings_description( $description, $pannel_id ) {

	if ( 'dslc_woo_license_main' === $pannel_id ) {

		$license = get_option( 'lcwoo_edd_license_key' );
		$status  = get_option( 'lcwoo_edd_license_status' );

		// Start output fetching.
		ob_start();
		?>
		<div class="wrap">

				<?php settings_fields( 'lcwoo_edd_license' ); ?>

				<table class="form-table">
					<tbody>
						<tr valign="top">
							<th scope="row" valign="top">
								<?php _e( 'License Key' ); ?>
							</th>
							<td>
								<input id="lcwoo_edd_license_key" name="lcwoo_edd_license_key" type="text" class="regular-text" value="<?php esc_attr_e( $license ); ?>" />
								<label class="description" for="lcwoo_edd_license_key"><?php _e( 'Enter your license key' ); ?></label>
							</td>
						</tr>
						<?php if ( false !== $license ) { ?>
							<tr valign="top">
								<th scope="row" valign="top">
									<?php _e( 'Activate License' ); ?>
								</th>
								<td>
									<?php if( $status !== false && $status == 'valid' ) { ?>
										<span style="color:green;"><?php _e( 'active' ); ?></span>
										<?php wp_nonce_field( 'lcwoo_edd_nonce', 'lcwoo_edd_nonce' ); ?>
										<input type="submit" class="button-secondary" name="lcwoo_edd_license_deactivate" value="<?php _e( 'Deactivate License' ); ?>"/>
									<?php } else {
										wp_nonce_field( 'lcwoo_edd_nonce', 'lcwoo_edd_nonce' ); ?>
										<input type="submit" class="button-secondary" name="lcwoo_edd_license_activate" value="<?php _e( 'Activate License' ); ?>"/>
									<?php } ?>
								</td>
							</tr>
						<?php } ?>
					</tbody>
				</table>

		<?php

		// Get the output and stop fetching.
		$description = ob_get_contents();
		ob_end_clean();
	}

	return $description;
}
add_filter( 'dslc_filter_section_description', 'lcwoo_add_settings_description', 10, 2 );
